import numpy as np

def apodize(radius, shape):
    """
    Create edges apodization tapper
    
    Parameters
    ----------
    nx, ny : integers
    size of the tapper
    radius : float
    radius must be lower than 1 and greater than 0.
    
    Returns
    -------
    
    tapper : numpy array ready to multiply on your image
    to apodize edges
    """
    ny = shape[0]
    nx = shape[1]

    if (radius >= 1) or (radius <= 0.):
        print('Error: radius must be lower than 1 and greater than 0.')
        return
        
    ni = np.fix(radius*nx)
    dni = int(nx-ni)
    nj = np.fix(radius*ny)
    dnj = int(ny-nj)
    
    tap1d_x = np.ones(nx)
    tap1d_y = np.ones(ny)
    
    tap1d_x[0:dni] = (np.cos(3. * np.pi/2. + np.pi/2.* (1.* np.arange(dni)/(dni-1)) ))
    tap1d_x[nx-dni:] = (np.cos(0. + np.pi/2. * (1.* np.arange(dni)/(dni-1)) ))
    tap1d_y[0:dnj] = (np.cos(3. * np.pi/2. + np.pi/2. * (1.* np.arange( dnj )/(dnj-1)) ))
    tap1d_y[ny-dnj:] = (np.cos(0. + np.pi/2. * (1.* np.arange(dnj)/(dnj-1)) ))
    
    tapper = np.zeros((ny, nx))
    
    for i in range(nx):
        tapper[:,i] = tap1d_y
                        
    for i in range(ny):
        tapper[i,:] = tapper[i,:] * tap1d_x
        
    return tapper


def padding(input, fact):                
    y, x = int(input.shape[0]*(1.+fact)), int(input.shape[1]*(1.+fact)) 
    
    width = input.shape[1]
    height = input.shape[0]
    
    output = np.zeros((y,x))
    
    xpos = np.int(x/2 - width/2)
    ypos = np.int(y/2 - height/2)
    
    output[ypos:height+ypos,xpos:width+xpos] = input                

    return output
    
        
def depad(input, fact):
    y, x = int(input.shape[0]/(1.+fact))+1, int(input.shape[1]/(1.+fact))+1 

    width = input.shape[1]
    height = input.shape[0]
    
    output = np.zeros((y,x))
    
    xpos = np.int(width/2 - x/2)
    ypos = np.int(height/2 - y/2)
    
    output = input[ypos:y+ypos,xpos:x+xpos]                

    return output



# -*- coding: utf-8 -*-
import numpy as np
import matplotlib.pyplot as plt
from scipy import integrate
from powerspectrum import PowerS
from fBms import fBmnd
from scipy import stats

plt.ion()
cm = plt.get_cmap('inferno')
cm.set_bad(color='black')
imkw = dict(origin='lower', interpolation='none', cmap=cm)

class StructF(object):
    def __init__(self, field, unit_length=1):
        super(StructF, self).__init__()
        self.field = field
        self.shape = field.shape
        self.unit_length = unit_length
        self.minval = 0.
        self.maxval = np.sqrt(self.shape[0]**2 + self.shape[1]**2)

        
    def shift_2D(self, data, dx, dy, constant=False):
        shifted_data = np.roll(data, dx, axis=1)
        if dx < 0:
            shifted_data[:, dx:] = constant
        elif dx > 0:
            shifted_data[:, 0:np.abs(dx)] = constant
            
        shifted_data = np.roll(shifted_data, dy, axis=0)
        if dy < 0:
            shifted_data[dy:, :] = constant
        elif dy > 0:
            shifted_data[0:np.abs(dy), :] = constant
        return shifted_data
    

    def shift_field(self, return_obj=False):
        if len(self.shape) == 2:
            self.dys = np.arange(0, self.shape[0], 1)
            self.dxs = np.arange(0, self.shape[1], 1)
            shifts = [[self.shift_2D(self.field, dy, dx, constant=np.nan) for dy in self.dys] for dx in self.dxs]
            
        else:
            print("Invalid shape encountered. Should be a 2D pr 3D array")

        self.shifts = shifts
        
        if return_obj:
            return shifts


    def mask_lag(self, l):
        lag_map = self.lag_map()
        return lag_map == l


    def shift_field_lag(self, l=None):
        mask = self.mask_lag(l)
        index = np.where(mask == True)
        return [self.shift_2D(self.field, index[0][i], index[1][i], constant=np.nan) for i in np.arange(len(index[0]))]            


    def diff_shift_field_lag(self, l=None):
        mask = self.mask_lag(l)
        index = np.where(mask == True)
        shifts = self.shift_field_lag(l)
        return [shift - self.field for shift in shifts]           

        
    def mean_diff_tab(self, shifts, order):
        self.dys = np.arange(0, self.shape[0], 1)
        self.dxs = np.arange(0, self.shape[1], 1)        
        mean_diff = [[np.nanmean(np.abs(shifts[i][j] - self.field)**order) for i in self.dys] for j in self.dxs]
        # mean_diff = [[np.nanmean((shifts[i][j] - self.field)**order) for i in self.dys] for j in self.dxs]
        return mean_diff
        

    def lag_map(self):
        return np.array([[np.sqrt(dy**2 + dx**2) for dy in np.arange(self.shape[0])] for dx in np.arange(self.shape[1])]).astype(int)


    def max_lag(self):
        lag_map = self.lag_map()
        return lag_map.max()
                

    def run(self, shifts=None, order=2, minval=None, maxval=None, bin_val=1, bin_type="logspace", nb_point=20):
        shifts = shifts or self.shift_field(return_obj=True)
        mean = self.mean_diff_tab(shifts=shifts, order=order)
        
        lag_map = [[np.sqrt(dy**2 + dx**2) for dy in np.arange(self.shape[0])] for dx in np.arange(self.shape[1])]

        # Get sorted radii
        flat_lag = np.array([item for sublist in lag_map for item in sublist])
        flat_mean = np.array([item for sublist in mean for item in sublist])
        
        ind = np.array(np.argsort(flat_lag))

        l_sorted = flat_lag[ind]
        mean_sorted = flat_mean[ind]

        # Get the integer part of the lag (bin size = 1)
        l_int = l_sorted.astype(int)
        deltal = l_int[1:] - l_int[:-1]  # Assumes all lag represented
        lind = np.where(deltal)[0]       # location of changed lag

        struct_f = [np.mean(mean_sorted[lind[i]+1:lind[i+1]+1]) for i in np.arange(len(lind)-1)]
        l = [np.mean(l_sorted[lind[i]+1:lind[i+1]+1]) for i in np.arange(len(lind)-1)]
        
        return struct_f, l

    def get_r_sf(self):
        lag_map = [[np.sqrt(dy**2 + dx**2) for dy in np.arange(self.shape[0])] for dx in np.arange(self.shape[1])]
        flat_lag = np.array([item for sublist in lag_map for item in sublist])        
        ind = np.array(np.argsort(flat_lag))
        l_sorted = flat_lag[ind]

        # Get the integer part of the lag (bin size = 1)
        l_int = l_sorted.astype(int)
        deltal = l_int[1:] - l_int[:-1]  # Assumes all lag represented
        lind = np.where(deltal)[0]       # location of changed lag

        l = [np.mean(l_sorted[lind[i]+1:lind[i+1]+1]) for i in np.arange(len(lind)-1)]

        return l 


    def integrand_SF_2(self, k, r, beta):
        return k**(-beta+1) * (1. - (np.sin(k*r) / (k*r)))

    def SF_2(self, r, beta):
        J = integrate.quad(self.integrand_SF_2, 0., np.inf, args=(r, beta))
        return 4. * np.pi * J[0]

    def sf2_from_gamma(self, rs, beta):
        return np.array([self.SF_2(r, np.abs(beta)) for r in rs])

    # def sf2_from_ps(self, unit_length=1):
    #     stat_ps = PowerS(self.field)
    #     sps1d, ks = stat_ps.sps1d(return_log=False)
    #     slope, intercept, r_value, p_value, std_err = stats.linregress(np.log10(ks),np.log10(sps1d))
    #     # rs = self.get_r_sf()
    #     rs = 1. / np.array(ks)
    #     SF2 = [self.SF_2(r, np.abs(slope)) for r in rs]
    #     return SF2, rs
        
if __name__ == '__main__':    
    shape = (128, 128)
    beta = 2.6
    field = fBmnd(beta, shape)
    stat = StructF(field)

    stop

    stat_ps = PowerS(field)
    sps1d, ks = stat_ps.sps1d(return_log=False)
    slope, intercept, r_value, p_value, std_err = stats.linregress(np.log10(ks),np.log10(sps1d))

    rs = 1. / np.array(ks)
    
    F2_from_SPS, rs_from_SPS = stat.sf2_from_ps()
    SF2, l = stat.run(order=2)
    SF3, l = stat.run(order=3)
    stop
    beta1, intercept1, r_value1, p_value1, std_err1 = stats.linregress(np.log10(rs),np.log10(np.abs(F2_from_SPS)))
    beta2, intercept2, r_value2, p_value2, std_err2 = stats.linregress(np.log10(l)[10:30],np.log10(np.abs(SF2)[10:30]))
    
    print "True", beta - 2.
    print "from SPS", beta1
    print "from map", beta2
    # print slope #FIXME

    #Test new ESS AM
    betas = np.arange(12)+2
    betas_K41 = [n/(n-1.) for n in betas[1:]]
    betas_ESS = [n/3. for n in betas]
    SF = [stat.run(order=b) for b in betas]

    slopes = []
    slopes_ESS = []
    for i in np.arange(len(betas)-1):
        slope, intercept, r_value, p_value, std_err = stats.linregress(np.log10(SF[i][0]),np.log10(SF[i+1][0]))
        slope_ESS, intercept, r_value, p_value, std_err = stats.linregress(np.log10(SF[1][0]),np.log10(SF[i][0]))
        slopes.append(slope)
        slopes_ESS.append(slope_ESS)

    er_K41 = (np.array(betas_K41[:11]) - np.array(slopes)) / np.array(betas_K41[:11]) * 100.
    er_ESS = (np.array(betas_ESS[:11]) - np.array(slopes_ESS)) / np.array(betas_ESS[:11]) * 100.
    



from .structuref import StructF
from .powerspectrum import PowerS
from .fBms import *
from .fluid import *
from .edges import *
from .gauss_beam import *
from .setup_plot import *
import projection as proj
import constant as cst
from velocity import lsr2helio
from .wolfire import wolfire
from powspec import powspec



import numpy as np
from astropy import units as u
from scipy.optimize import fsolve

class wolfire(object):
        def __init__(self, T=3000, n=0.5, phi=0.5, G0=1.7, Zd=1., zeta=1., Zg=1, fCII=1., Ac=1.4e-4, Ao=3.2e-4, Nw=1.e19):
                super(wolfire, self).__init__()
                self.T = T
                self.n = n
                self.phi = phi
                self.G0 = G0
                self.Zd = Zd
                self.Zg = Zg
                self.zeta = zeta
                self.fCII = fCII
                self.Ac = Ac #wolfire 03
                self.Ao = Ao #wolfire 03
                self.Nw = Nw

        def ne(self):
            # return 0.021 *u.cm**-3
            return 2.4e-3 * self.zeta**0.5 * (self.T/100.)**0.25 * (self.G0/1.7)**0.5 * self.Zd**-0.5 * self.phi**-1 *u.cm**-3
            # return 2.4e-3 * (self.T/100.)**0.25 * (self.G0 / 1.7)**0.5 / self.phi *u.cm**-3 #HERACLES

        def eff03(self):
            ne = self.ne().value
            return 4.9e-2 / (1. + 4.e-3*(self.G0*self.T**0.5/ne/self.phi)**0.73) 
            + ((3.7e-2*(self.T/1.e4)**0.7)/(1. + 2.0e-4*(self.G0*self.T**0.5/ne/self.phi)))

        def eff95(self):
            param = self.G0*self.T**0.5/self.ne().value
            return 4.9e-2 / (1. + (param/1925.)**0.73) 
            + ((3.7e-2*(self.T/1.e4)**0.7)/(1. + (param/5000.))) #HERACLES

        def effBT(self):
            param = self.G0*self.T**0.5/self.ne().value
            return 4.87e-2 / (1. + 4.e-3*(param)**0.73) 
            + ((3.65e-2*(self.T/1.e4)**0.7)/(1. + 2.e-4*param))

        def heat(self):
            # return 1.e-24 * self.n * self.eff95() * self.G0 * u.erg * u.cm**-3 * u.s**-1 #HERACLES
            return 1.3e-24 * self.n * self.eff03() * self.G0 * u.erg * u.s**-1 * u.cm**-3

        def cool_CII(self):
            ne = self.ne().value
            return 2.54e-14 * self.Ac * (2.8e-7 * ne / self.n * (self.T/100.)**-0.5 + 8.e-10) * np.exp(-92/self.T) * u.erg * u.cm**3 * u.s**-1
            # return (92. * 1.38e-16 * 2. * (2.8e-7* ((self.T/100.)**(-0.5))*ne/self.n + 8.e-10*((self.T/100.)**(0.07))) * 3.5e-4 * 0.4 * np.exp(-92./ self.T)) * u.erg * u.cm**3 * u.s**-1 #From HERACLES so from Karl Joulain thesis

        def Tn(self,n):
            return self.T/10.**n

        def Omega(self):
            return 1.80 + 0.484*self.Tn(4) + 4.01*self.Tn(4)**2 - 3.39*self.Tn(4)**3

        def cool_CII95(self):
            ne = self.ne().value
            gamma = 8.86e-10
            gamma_e = 2.1e-7 * self.Tn(2)**-0.5 * self.Omega()
            return 2.54e-14 * self.Ac * self.fCII * (gamma + gamma_e*(ne/self.n)) * np.exp(-92/self.T) * u.erg * u.cm**3 * u.s**-1

        def cool_OI(self):
            # O = 4.5e-4 #HERACLES
            return 1.e-26 * self.Ao * self.T**0.5 * (24. * np.exp(-228./self.T) + 7.*np.exp(-326./self.T)) * u.erg * u.cm**3 * u.s**-1
            # return 1.e-26 * O * self.T**0.5 * (24. * np.exp(-228./self.T) + 7.*np.exp(-326./self.T)) * u.erg * u.cm**3 * u.s**-1 #HERACLES

        def cool_Ly(self):
            ne = self.ne().value
            return 7.3e-19 * ne / self.n * np.exp(-118400./self.T) * u.erg * u.cm**3 * u.s**-1 #HERACLES

        def cool_rec03(self):
            ne = self.ne().value
            return 4.65e-30 * self.T**0.94 * (self.G0 * self.T**0.5 / ne / self.phi)**(0.74/self.T**0.068) * ne / self.n * self.phi * u.erg * u.cm**3 * u.s**-1

        def cool_rec95(self):
            ne = self.ne().value
            return 4.65e-30 * self.T**0.94 * (self.G0 * self.T**0.5 / ne)**(0.74/(self.T**0.068)) * ne / self.n * u.erg * u.cm**3 * u.s**-1

        def cool(self):
            return (self.n*u.cm**-3)**2 * (self.cool_CII95() + self.cool_OI() + self.cool_Ly() + self.cool_rec03())

        def fsix(self,p2):
            return 0.990 - (2.74e-3*p2) + (1.13e-3*p2**2)

        def heatXR(self):
            p1=np.log10(self.Nw/1.e18)
            p2=np.log10(self.ne().value/self.n)
            XR = self.fsix(p2)*(-26.5 -(0.920*p1) + (5.89e-2*p1**2)) + self.fsix(p2)*0.96*np.exp(-((p1-0.38)/0.87)**2)
            return self.n * 10.**XR * u.erg * u.cm**-3 * u.s**-1 #FIXME
            
        def loss(self):
            return self.heat() + self.heatXR() - self.cool()

        # def PminJenkins(self):
        #         return 5730. * (Zd * 1.)  * (Zd / (1. + 2.08*(Zd * 1.)**0.365)) #FIXME I/I0

if __name__ == '__main__':
        
        T = np.logspace(np.log10(1), np.log10(10000), 100)
        n = np.logspace(np.log10(0.01), np.log10(1000), 100)
        
        core = wolfire(T=T,n=1)
        cooling = core.cool()
        heating = core.heat()
                
        Teq = np.array([fsolve(lambda x: wolfire(T=x, n=n[i], G0=1.7, phi=0.5, Nw=1.e19).loss(), 1)[0] for i in np.arange(len(n))])
        Teq_low = np.array([fsolve(lambda x: wolfire(T=x, n=n[i], G0=0.5*1.7, phi=0.5, Nw=1.e19).loss(), 1)[0] for i in np.arange(len(n))])
        Teq_high = np.array([fsolve(lambda x: wolfire(T=x, n=n[i], G0=2.*1.7, phi=0.5, Nw=1.e19).loss(), 1)[0] for i in np.arange(len(n))])

        heat =  wolfire(T=Teq,n=n, G0=1.7, phi=0.5, Nw=1.e19).heat() / n
        heatXR =  wolfire(T=Teq,n=n, G0=1.7, phi=0.5, Nw=1.e19).heatXR() / n 
        cool_CII95 =  wolfire(T=Teq,n=n, G0=1.7, phi=0.5, Nw=1.e19).cool_CII95() * n
        cool_Ly =  wolfire(T=Teq,n=n, G0=1.7, phi=0.5, Nw=1.e19).cool_Ly() * n
        cool_rec03 =  wolfire(T=Teq,n=n, G0=1.7, phi=0.5, Nw=1.e19).cool_rec03() * n
        cool_OI =  wolfire(T=Teq,n=n, G0=1.7, phi=0.5, Nw=1.e19).cool_OI() * n

        stop

        fig = plt.figure(figsize=(12, 8))
        ax = fig.add_subplot(111)
        ax.set_yscale('log')
        ax.set_xscale('log')
        ax.set_xlim([0.01,1000])
        ax.set_ylim([1.e1,1.e5])
        ax.plot(n,n*Teq_low, "--k", label=r"G$_0$=0.87", linewidth=2.)
        ax.plot(n,n*Teq, color="k", label=r"G$_0$=1.7", linewidth=2.)
        ax.plot(n,n*Teq_high, "-.k", label=r"G$_0$=3.4", linewidth=2.)
        ax.set_ylabel(r'$P/k_B$ (K.cm$^{-3}$)',  fontsize = 16)
        ax.set_xlabel(r'$n$ (cm$^{-3}$)', fontsize = 16)
        plt.legend(loc = 2, numpoints = 1)
        leg = plt.gca().get_legend()
        ltext  = leg.get_texts()
        plt.setp(ltext, fontsize = 'small')        
        plt.savefig('wolfire_solar.png', format='png', bbox_inches='tight', pad_inches=0.02)
        


        
    




import numpy as np

def gauss_beam(sigma, shape, FWHM=False):
        '''
	Return a Gaussian beam at the center of an image
	
	Parameters
	----------
	sigma: float 
		Standard deviation (sigma) of the Gaussian function
	nx, ny: integers
		Size of the output image
	FWHM: bool
		If True, the sigma parameter becomes the Full Width Half Max
		
	Return
	----------
	Gaussian beam at the center of the image
	'''
	ny, nx = shape
	X=np.arange(nx)
	Y=np.arange(ny)
	ymap,xmap=np.meshgrid(X,Y)
	
	if (nx % 2) == 0:
		xmap = xmap - (nx)/2.
	else:
		xmap = xmap - (nx-1.)/2.

	if (ny % 2) == 0:
		ymap = ymap - (ny)/2.
	else:
		ymap = ymap - (ny-1.)/2.
                
        map = np.sqrt(xmap**2.+ymap**2.)
        
	if FWHM == True:	
		sigma = sigma / (2.*np.sqrt(2.*np.log(2.)))
                
        gauss = np.exp(-0.5*(map)**2./sigma**2.)
		
	return gauss



import numpy as np
import matplotlib.pyplot as plt
from scipy import stats
from scipy import ndimage
from statsmodels import robust
from fBms import fBmnd
from fBms import Pkgen
from edges import *

plt.ion()

class PowerS(object):
        def __init__(self, field, unit_length=1):
                super(PowerS, self).__init__()
                self.field = field
                self.shape = field.shape
                self.unit_length = unit_length

        def azimuthalAverage(self, image, unit_length=1., return_std=False):
                """
                Calculate the azimuthally averaged radial profile.
                
                image - The 2D image
                center - The [x,y] pixel coordinates used as the center. The default is 
                None, which then uses the center of the image (including 
                fracitonal pixels).
                
                """
                if self.shape[0] != self.shape[0]:
                        "Warning : Two dimensions are not the same / Be careful with the 2D Spatial Power Spectrum"
                        
                # Calculate the indices from the image
                y, x = np.indices(self.shape)
                
                center = np.array([(x.max()-x.min())/2.0, (x.max()-x.min())/2.0])
                
                r = np.hypot(x - center[0], y - center[1])

                # Get sorted radii
                ind = np.argsort(r.flat)
                r_sorted = r.flat[ind]
                i_sorted = image.flat[ind]
                
                # Get the integer part of the radii (bin size = 1)
                r_int = r_sorted.astype(int)
                deltar = r_int[1:] - r_int[:-1]  # Assumes all radii represented
                rind = np.where(deltar)[0]       # location of changed radius

                vals = [i_sorted[rind[i]+1:rind[i+1]+1] for i in np.arange(len(rind)-1)]
                azi_average = [np.median(val) for val in vals]
                std = [robust.mad(val) for val in vals]
                logstd = (np.array(std)) / np.array(azi_average) / np.log(10.)

                all_k = [np.fft.fftshift(np.fft.fftfreq(s, d=unit_length)) for s in self.shape[:-1]] + \
                        [np.fft.fftshift(np.fft.fftfreq(self.shape[-1], d=unit_length))]
                
                kgrid = [np.zeros(self.shape) for _ in range(len(self.shape))]
                
                for i, kg in enumerate(kgrid):
                        sl = [slice(None) if j == i
                              else None
                              for j in range(len(self.shape))
                        ]
                        kg[:] = all_k[i][sl]
                        
                k2 = np.sqrt(np.sum([k**2 for k in kgrid], axis=0))
                k2_sorted = k2.flat[ind]
                vals_k2 = [k2_sorted[rind[i]+1:rind[i+1]+1] for i in np.arange(len(rind)-1)]
                ks = [np.mean(val) for val in vals_k2]
                
                idx = np.where(np.array(ks) < 0.5)[0]                
                
                # if return_log == True:
                #         return np.log10(sps1d), np.log10(ks) #logstd
                # else:
                # print azi_average[:idx[-1]]
                if return_std == True:
                        return np.array(azi_average[:idx[-1]]), std[:idx[-1]]
                else:
                        return np.array(azi_average[:idx[-1]])

        def azimuthalAverage_nd(self, field, unit_length=1.):
                idx = np.indices(self.shape)                
                center = np.array(self.shape)/2
                distance = (idx.transpose() - center).transpose()
                r = np.sqrt(np.sum(distance**2,0))

                # Get sorted radii
                ind = np.argsort(r.flat)
                r_sorted = r.flat[ind]
                i_sorted = field.flat[ind]
                
                # Get the integer part of the radii (bin size = 1)
                r_int = r_sorted.astype(int)
                deltar = r_int[1:] - r_int[:-1]  # Assumes all radii represented
                rind = np.where(deltar)[0]       # location of changed radius

                vals = [i_sorted[rind[i]+1:rind[i+1]+1] for i in np.arange(len(rind)-1)]
                azi_average = [np.mean(val) for val in vals]
                std = [robust.mad(val) for val in vals]
                logstd = (np.array(std)) / np.array(azi_average) / np.log(10.)

                # Compute the k grid
                ks = [np.fft.fftshift(np.fft.fftfreq(s, d=unit_length)) for s in self.shape]
                
                if len(ks) == 3 : 
                        kgrid = np.meshgrid(ks[1],ks[0],ks[2])
                else:
                        kgrid = np.meshgrid(*ks)
                        
                knorm = np.sqrt(np.sum(np.power(kgrid,2), axis=0))
                        
                knorm_sorted = knorm.flat[ind]
                vals_knorm = [knorm_sorted[rind[i]+1:rind[i+1]+1] for i in np.arange(len(rind)-1)]
                kmean = [np.mean(val) for val in vals_knorm]

                kcut = np.where(np.array(kmean) < 0.5)[0]                                
                                
                return np.array(azi_average[:kcut[-1]])
                        

        def sps2d(self):
                # Return square modulus of complex number
                fftfield = np.fft.fft2(self.field)
                shiftfftfield = np.fft.fftshift(fftfield)
        
                return np.power(np.abs(shiftfftfield), 2) 

        def spsnd(self):
                # Return square modulus of complex number
                fftfield = np.fft.fftn(self.field)
                shiftfftfield = np.fft.fftshift(fftfield)
        
                return np.power(np.abs(shiftfftfield), 2) 


        # def sps1d(self, return_log=False):
        #         psnd = self.spsnd()
        #         sps1d = self.azimuthalAverage_nd(psnd)
        #         ks = self.get_ks()
                
        #         return sps1d

        def sps1d(self, return_log=False):
                ps2d = self.sps2d()
                sps1d, std = self.azimuthalAverage(ps2d, return_std=True)
                ks = self.get_ks()
                
                return sps1d, std
        

        # def get_ks(self, unit_length=1):                
        #         # Compute the k grid
        #         ks = [np.fft.fftshift(np.fft.fftfreq(s, d=unit_length)) for s in self.shape]
                
        #         if len(ks) == 3 : 
        #                 kgrid = np.meshgrid(ks[1],ks[0],ks[2])
        #         else:
        #                 kgrid = np.meshgrid(*ks)
                        
        #         knorm = np.sqrt(np.sum(np.power(kgrid,2), axis=0))

        #         k = self.azimuthalAverage_nd(knorm)
        #         return np.array(k)

        def get_ks(self, unit_length=1):                
                all_k = [np.fft.fftshift(np.fft.fftfreq(s, d=unit_length)) for s in self.shape[:-1]] + \
                        [np.fft.fftshift(np.fft.fftfreq(self.shape[-1], d=unit_length))]
                
                kgrid = [np.zeros(self.shape) for _ in range(len(self.shape))]
                
                for i, kg in enumerate(kgrid):
                        sl = [slice(None) if j == i
                              else None
                              for j in range(len(self.shape))
                        ]
                        kg[:] = all_k[i][sl]
                        
                k2 = np.sqrt(np.sum([k**2 for k in kgrid], axis=0))
                ks = self.azimuthalAverage(k2)
                return np.array(ks)

                
        
if __name__ == '__main__':
        shape = (256,256)
        field = fBmnd(shape, Pkgen(3,0.01,0.45), seed=31, unit_length=1)

        stat = PowerS(field)
        sps1d, std = stat.sps1d()
        ks = stat.get_ks(unit_length=1)
        
        slope, intercept, r_value, p_value, std_err = stats.linregress(np.log10(ks[3:]),np.log10(sps1d[3:]))
        
        print("Beta = " + str(slope))

        stop
                
        fig = plt.figure(0)
        ax = fig.add_subplot(111)
        ax.set_yscale('log')
        ax.set_xscale('log')
        ax.plot(ks, sps1d, '.k', markersize=2)

        # ax.plot(np.log10(np.arange(len(logsps1d))+1), logsps1d, '.k', markersize=2, label='TOT')
        # ax.fill_between(np.log10(np.arange(len(logsps1d))+1), logsps1d-logstd, logsps1d+logstd,
        #                                  alpha=0.2, edgecolor='k', facecolor='k', linewidth=0)

        # fig = plt.figure()
        # ax = fig.add_subplot(111)
        # ax.errorbar(np.log10(np.arange(len(sps1d))+1), np.log10(sps1d), yerr=err, fmt=".k")
        # ax.set_yscale('log', nonposy="clip")
        # ax.set_xscale('log', nonposx="clip")
        # ax.errorbar(np.arange(len(sps1d)), sps1d, yerr=np.array(stdp)/2., fmt=".")



import os
import numpy as np
import colorcet as cc
import matplotlib
import matplotlib.pyplot as plt

cm_coolwarm = cc.cm["coolwarm"]
cm_coolwarm.set_bad(color='white')
cm_coolwarm.set_under(color='white')
imkw_coolwarm = dict(origin='lower', interpolation='none', cmap=cm_coolwarm)

cm_inferno = plt.get_cmap('inferno')
cm_inferno.set_bad(color='black')
cm_inferno.set_under(color='black')
imkw_inferno = dict(origin='lower', interpolation='none', cmap=cm_inferno)

cm_afmhot = plt.get_cmap('afmhot')
cm_afmhot.set_bad(color='black')
imkw_afmhot = dict(origin='lower', interpolation='none', cmap=cm_afmhot)

cm_viridis = plt.get_cmap('viridis')
cm_viridis.set_bad(color='black')
imkw_viridis = dict(origin='lower', interpolation='none', cmap=cm_viridis)

cm_bone = plt.get_cmap('bone')
cm_bone.set_bad(color='black')
cm_coolwarm.set_under(color='black')
imkw_bone = dict(origin='lower', interpolation='none', cmap=cm_bone)

cm_cubehelix = plt.get_cmap('cubehelix')
cm_cubehelix.set_bad(color='black')
imkw_cubehelix = dict(origin='lower', interpolation='none', cmap=cm_cubehelix)

# cm_planck = matplotlib.colors.ListedColormap(np.loadtxt("Planck_Parchment_RGB.txt")/255.)
# cm_planck.set_bad("white")
# cm_planck.set_under("white")
# imkw_planck = dict(origin='lower', interpolation='none', cmap=cm_planck)





import numpy as np

m_h = 1.6737236e-27 #kg                              

# def kinetic_T(sigma_obs, mach):
#     sigma_therm = sigma_obs / np.sqrt(1. + mach**2.)
#     return m_h * (sigma_therm*1.e3)**2 / const.k_B.value # K

def kinetic_T(sigma_obs, mach):
    return 121. * sigma_obs**2. / ((mach**2./4.2) + 1)

def PDF2mach(sigma,zeta,D):
    b = 1. + ((1./D) - 1.)*zeta
    return np.sqrt((np.exp(sigma**2.) - 1.) / b**2.)

if __name__ == '__main__':    
    print(kinetic_T(8.,0.))
    print(PDF2mach(0.40,0.2,3))


def v_r(r):
    #Define constant in kpc and solar mass                                                                                                                                                                                                                                            
    G  = 4.302e-6
    a1 = 0.
    b1 = 0.495
    M1 = 2.05e10
    a2 = 7.258
    b2 = 0.520
    M2 = 25.47e10

    return r * np.sqrt((G*M1 / (r**2.+(a1+b1)**2.)**(3./2.))
                       + (G*M2 / (r**2.+(a2+b2)**2.)**(3./2.)))

def fct_r(l, b, r0, d):
    return r0 * np.sqrt( np.cos(b)**2 * (d/r0)**2 - (2.*np.cos(b)*np.cos(l)*(d/r0)) + 1. )

def fct_vlsr(r, r0, l, b):
    return ( r0 * v_r(r) / r - v_r(r0) ) * np.sin(l) * np.cos(b)

def vlimlsr(l, b):
    '''l in [0, 2pi] / b in [-pi/2, pi/2]'''
    # Define constant / unit kpc and km/s                                                                                                                                                                                                                                             
    r0 = 8.5
    # Disk geometry                                                                                                                                                                                                                                                                   
    z1 = 1.
    z2 = 3.
    rmax = 26.

    d     = 0.
    z     = d * np.sin(b)
    zmax  = 1.
    r     = 0.
    save  = []
    count = 0
    test  = False

    while r < rmax :
        if np.abs(z) > zmax :
            test = True
        if test == True : break
        r = fct_r(l, b, r0, d)
        if r != 0. :
            vlsr = fct_vlsr(r, r0, l, b)
            if count != 0 :
                save.append(vlsr)
            if r <= r0 :
                zmax = z1
            if r > r0 :
                zmax = z1 + (z2 - z1) * ( (r/r0 - 1.)**2 / 4. )
        d += 0.1
        z = d * np.sin(b)
        count += 1
    return np.min(save), np.max(save)



import numpy as np
from astropy import wcs

def wcs2D(hdr):
    w = wcs.WCS(naxis=2)
    w.wcs.crpix = [hdr['CRPIX1'], hdr['CRPIX2']]
    w.wcs.cdelt = np.array([hdr['CDELT1'], hdr['CDELT2']])
    w.wcs.crval = [hdr['CRVAL1'], hdr['CRVAL2']]
    w.wcs.ctype = [hdr['CTYPE1'], hdr['CTYPE2']]
    return w



import numpy as np
import matplotlib.pyplot as plt
from astropy.io import fits
from scipy import stats
from scipy import ndimage
from statsmodels import robust
from fBms import fBmnd
from scipy.stats import moment

from edges import *

import turbulence as tb 

plt.ion()

shape=(1024,256,256)
unit_length=1.

# field = fBmnd(3., shape)

ks = [np.fft.fftshift(np.fft.fftfreq(s, d=unit_length)) for s in shape]
kx, ky, kz = np.meshgrid(ks[1],ks[0],ks[2])
# kx = ellip * (kx * costheta - ky * sintheta)
# ky = kx * sintheta + ky * costheta
k = np.sqrt(kx**2 + ky**2 + kz**2)

rdm3D = np.random.normal(size=shape) 
phase = np.angle(np.fft.fftn(rdm3D))
power_k = np.where(k == 0, 0, np.sqrt(np.power(k, -3.)))
imfft = np.zeros(shape, dtype=complex)
imfft.real = power_k * np.cos(phase)
imfft.imag = power_k * np.sin(phase)
field3D = np.fft.ifftn(np.fft.ifftshift(imfft)).real

field3D += 5*np.std(field3D)
field2D = np.nansum(field3D,0)

field3Dnorm = field3D / np.nanmean(field3D)
field2Dnorm = field2D / np.nanmean(field2D)

sig3D = np.sqrt(moment(field3Dnorm.ravel(), moment=2, nan_policy='omit'))
sig2D = np.sqrt(moment(field2Dnorm.ravel(), moment=2, nan_policy='omit'))

R = sig2D**2. / sig3D**2.

print "R = ", R

# ###
# field3D = field3D[:500,:500,:500] # !
# shape = (500,500,500)
# ks = [np.fft.fftshift(np.fft.fftfreq(s, d=unit_length)) for s in shape]
# kx, ky, kz = np.meshgrid(ks[1],ks[0],ks[2])
# k = np.sqrt(kx**2 + ky**2 + kz**2)
# ###

#mask WNM from Saury
#3D Open data                                                                                                                                                                                                                                                                                             
path_simu = '/data/amarchal/ROHSA_paper/data/Saury2014/'
hdu_list_T = fits.open(path_simu + 'T_016_subgrid_256.fits')
T_cube = hdu_list_T[0].data

idx_wnm = T_cube < 5000
fv = float(len(T_cube[idx_wnm])) / len(T_cube.ravel())

field3D_f = np.copy(field3D)
field3D_f[idx_wnm] = np.nan  
field2D_f = np.nansum(field3D_f,0)

field3Dnorm_f = field3D_f / np.nanmean(field3D_f)
field2Dnorm_f = field2D_f / np.nanmean(field2D_f)

sig3D_f = np.sqrt(moment(field3Dnorm_f.ravel(), moment=2, nan_policy='omit'))
sig2D_f = np.sqrt(moment(field2Dnorm_f.ravel(), moment=2, nan_policy='omit'))

R_f = sig2D_f**2. / sig3D_f**2.

print "R_f = ", R_f

stop

#######################################
stat = tb.PowerS(field2D_f)
sps1d = stat.sps1d()
ksps1d = stat.get_ks(unit_length=1)

slope, intercept, r_value, p_value, std_err = stats.linregress(np.log10(ksps1d[3:]),np.log10(sps1d[3:])) 
model = 10.**(slope * np.log10(k) + intercept)
# model = 10.**(-3. * np.log10(k))
model[shape[0]/2,shape[1]/2,shape[2]/2] = np.nan
R_theo = np.nansum(model[shape[0]/2]) / np.nansum(model)

field3Dnorm = field3D / np.mean(field3D)
field2Dnorm = field2D / np.mean(field2D)

sig3D = np.sqrt(moment(field3Dnorm.ravel(), moment=2, nan_policy='propagate'))
sig2D = np.sqrt(moment(field2Dnorm.ravel(), moment=2, nan_policy='propagate'))

R = sig2D**2. / sig3D**2.





import numpy as np
import matplotlib.pyplot as plt
import scipy.spatial as sp
import healpy as hp
from fBms import fBmnd
from astropy.io import fits

def appendSpherical_np(xyz):
    ptsnew = np.hstack((xyz, np.zeros(xyz.shape)))
    xy = xyz[:,0]**2 + xyz[:,1]**2
    ptsnew[:,3] = np.sqrt(xy + xyz[:,2]**2)
    ptsnew[:,4] = -(np.arctan2(np.sqrt(xy), xyz[:,2])-(np.pi/2.)) # for elevation angle defined from Z-axis down
    ptsnew[:,5] = np.arctan2(xyz[:,1], xyz[:,0])
    return ptsnew

def find_nearest(array, value):               
    array = np.asarray(array)
    idx = (np.abs(array - value)).argmin()
    return array[idx]

if __name__ == '__main__':
    path = "/data/amarchal/LUDOVIC/data/"
    hdu = fits.open(path+"stilism_GAIAdr2_mapJL.fits")
    hdr = hdu[0].header
    cube=hdu[0].data

    dim_max = np.max(cube.shape)

    center = np.array([cube.shape[0]/2,cube.shape[1]/2,cube.shape[2]/2])

    x = np.arange(cube.shape[0])
    y = np.arange(cube.shape[1])
    z = np.arange(cube.shape[2])

    X,Y,Z=np.meshgrid(x,y,z)

    data=np.vstack((X.ravel(),Y.ravel(),Z.ravel())).T

    diff=10
    bins = np.arange(1,dim_max,diff)
    # tree=sp.cKDTree(data)
    # mask=[tree.query_ball_point(center, shell) for shell in bins]
    # shells=[list(set(mask[int(i)])^set(mask[int(i-1)])) for i in np.linspace(1,len(mask)-1,len(mask)-1)]
    # convert=[appendSpherical_np(data[shell]-center) for shell in shells]

    convert=appendSpherical_np(data-center)
    r = convert[:,3]
    l = np.degrees(convert[:,5]+np.pi)
    b = np.degrees(convert[:,4])
    ext = cube.ravel()
    
    nside=64
    
    reso = np.degrees(hp.nside2resol(nside))

    lhp, bhp = hp.pix2ang(nside, np.arange(hp.nside2npix(nside)), lonlat=True)

    idxr = np.where((r > 55) & (r < 65))[0]

    idx =[np.where((l[idxr]>lhp[i]-reso/2.) & (l[idxr]<lhp[i]+reso/2.) & (b[idxr]>bhp[i]-reso/2.) & (b[idxr]<bhp[i]+reso/2.)) for i in np.arange(len(lhp))]

    hpmap = np.zeros(hp.nside2npix(nside), dtype=np.float)
    hpmap = [np.nanmean(ext[idx[i]]) for i in np.arange(len(lhp))]

    #Exemple shell
    idx = np.where((r > 0) & (r < diff))[0]
    npix = len(idx)
    nsides = 2 ** np.arange(12)
    nside = find_nearest(nsides,np.sqrt(npix/12))

    hpmap = np.zeros(hp.nside2npix(nside), dtype=np.float)
    indices = hp.ang2pix(nside, l[idx], b[idx], lonlat=True)
    hpmap[indices] += ext[indices]
    
    
    

    



import numpy as np
import matplotlib.pyplot as plt
from matplotlib.ticker import ScalarFormatter

def pdf_log(array, linewidth=2., color="c", label=None, xlabel=None, save=False, log=True, bins=None):
    if bins is None: bins = np.logspace(np.log10(np.nanmin(array)), np.log10(np.nanmax(array)), 200)  
    fig = plt.figure(0, figsize=(10, 10))
    ax = fig.add_subplot(111)
    ax.set_xscale('log')
    ax.set_yscale('log')
    ax.xaxis.set_major_formatter(ScalarFormatter())
    ax.yaxis.set_major_formatter(ScalarFormatter())
    ax.hist(array, bins=bins, log=log, histtype='step', color=color, normed=False, label=label, linewidth=linewidth)
    ax.set_xlim([np.min(array), np.max(array)])                                                                                                                                                                                                                                   
    if xlabel is not None: ax.set_xlabel(xlabel, fontsize = 16) 
    ax.set_ylabel(r'$PDF$', fontsize = 16)
    plt.legend(loc = 1, numpoints = 1)
    leg = plt.gca().get_legend()
    ltext  = leg.get_texts()
    plt.setp(ltext, fontsize = 'small')
    if save == True : plt.savefig(path_plot + 'PDF.png', format='png')   

if __name__ == '__main__':
    print("toto")
    xlabel = r'$NHI$'
    
    mu, sigma = 2, 0.1
    array = np.random.normal(mu, sigma, 1000)
    pdf_log(array)



import numpy as np

def lsr2helio(l, b, vlsr, reverse=False):
    c1=-10.27
    c2=15.32
    c3=7.74

    ut=-np.cos(b)*np.cos(l)
    vt=np.cos(b)*np.sin(l)
    wt=np.sin(b)
    s=ut*c1+vt*c2+wt*c3
    if reverse==False:
        vhelio=vlsr-s
        print vhelio
        return vhelio
    else:
        vhelio=vlsr+s
        print vhelio
        return vhelio




import numpy as np

def powspec(image, reso=1, autocorr=False, **kwargs):
	"""
	Calculate the power spectrum of a 2D image

	Parameters
	----------
	image : array_like
		Input array, must 2-dimentional and real
	reso : float, optional
		Resolution of the image in pixel^-1
		
	autocorr : If True, calculate the autocorrelation function
		instead of the power spectrum. (False by default)
		
	Optional keywords
	------------------
	im2 : array_like
		Second input array of the same size than image.
		If called, the function returns the cross-spectrum
		between 'image' and 'im2'

	Returns
	-------

	tab_k : Array of spatial scales used for the decomposition
	spec_k: The power spectrum
	"""
	
	na=float(image.shape[1])
	nb=float(image.shape[0])
	nf=np.max(np.array([na,nb]))

	k_crit = nf/2    
	k_min = 1

	bins = np.arange(k_crit+1)

	#Fourier transform & 2D power spectrum
	#---------------------------------------------

	imft=np.fft.fft2(image)
	
	if 'im2' in kwargs:
		im2 = kwargs.get('im2')
		im2ft = np.fft.fft2(im2)
		ps2D = imft * np.conj(im2ft) / (na*nb)
	else:
		ps2D = np.abs( imft )**2 / (na*nb)

	del imft
    
	if autocorr == True:
		ps2D = np.fft.ifft2(ps2D).real

	#Set-up kbins
	#---------------------------------------------

	x=np.arange(na)
	y=np.arange(nb)
	x,y=np.meshgrid(x,y)

	if (na % 2) == 0:
		x = (1.*x - ((na)/2.) ) / na
		shiftx = (na)/2.
	else:
		x = (1.*x - (na-1)/2.)/ na
		shiftx = (na-1.)/2.+1

	if (nb % 2) == 0:
		y = (1.*y - ((nb/2.)) ) / nb
		shifty = (nb)/2.
	else:
		y = (1.*y - (nb-1)/2.)/ nb
		shifty = (nb-1.)/2+1

	k_mat = np.sqrt(x**2 + y**2)
	k_mat = k_mat * nf 
	#k_mod = np.round(np.fft.fftshift(k_mat),decimals=0)
	
	k_mat= np.roll(k_mat,int(shiftx), axis=1)
	k_mat= np.roll(k_mat,int(shifty), axis=0)
	k_mod = np.round(k_mat,decimals=0)

	hval, rbin = np.histogram(k_mod,bins=bins)

	#Average values in same k bin
	#---------------------------------------------

	kval = np.zeros(np.int(k_crit))
	kpow = np.zeros(np.int(k_crit))

	for j in range(np.int(k_crit)):

		kval[j] = np.sum(k_mod[k_mod == np.float(j)]) / hval[j]
		kpow[j] = np.sum(ps2D[k_mod == np.float(j)]) / hval[j]

	spec_k = kpow[1:np.size(hval)-1]

	if autocorr == False:
		tab_k = kval[1:np.size(hval)-1] / (k_crit * 2.* reso)
	else:
		tab_k = kval[1:np.size(hval)-1] * reso
    
	return tab_k, spec_k


# -*- coding: utf-8 -*-                                                                                                                                                                                                                             
from setuptools import setup, find_packages

with open('/home/amarchal/library/marchalib/Readme.md') as f:
        readme = f.read()

with open('/home/amarchal/library/marchalib/LICENSE') as f:
    license = f.read()

setup(
    name='marchalib',
    version='0.1.5',
    description='A. Marchal library',
    long_description=readme,
    classifiers=[
        'Development status :: 1 - Alpha',
        'License :: CC-By-SA2.0',
        'Programming Language :: Python',
        'Topic :: Data Analysis'
    ],
    author='Antoine Marchal',
    author_email='amarchal@cita.utoronto.ca',
    url='',
    license=license,
    packages=find_packages(exclude=('tests', 'docs')),
    install_requires=[
            'numpy',
	    'statsmodels',
	    'scipy',
	    'astropy',
	    'colorcet',
    ],
    include_package_data=True
)



# -*- coding: utf-8 -*-
import numpy as np
import six

def fBmnd(shape, power_spectrum, unit_length=1, seed=None, statistic=np.random.normal, fft=np.fft, fft_args=dict()):
    """
    Generates a field given a stastitic and a power_spectrum.
    author: A. Marchal based on FieldGenerator (C. Cadiou) and MAMDLIB 

    Parameters
    ----------
    statistic: callable
        A function that takes returns a random array of a given signature,
        with signature (s) -> (B) with s == B.shape

    shape: tuple
        The shape of the output field

    unit_length: float
        How much physical length represent 1pixel. For example a value of 10
        mean that each pixel stands for 10 physical units. It has the
        dimension of a physical_unit/pixel.

    fft: a numpy-like fft API

    fft_args: array
        a dictionary of kwargs to pass to the FFT calls

    Returns:
    --------
    field: a real array of shape `shape` following the statistic
        with the given power_spectrum
    """

    if seed is not None: np.random.seed(seed)

    if not six.callable(statistic):
        raise Exception('`statistic` should be callable')

    # Draw a random sample
    normal = statistic(size=shape)

    # Compute the FFT of the field and take the phase
    phase = np.angle(fft.fftn(normal, **fft_args))

    try:
        fftfreq = fft.fftfreq
    except:
        # Fallback on numpy for the frequencies
        fftfreq = np.fft.fftfreq

    # Compute the k grid
    ks = [np.fft.fftshift(np.fft.fftfreq(s, d=unit_length)) for s in shape]

    if len(ks) == 3 : 
        kgrid = np.meshgrid(ks[1],ks[0],ks[2])
    else:
        kgrid = np.meshgrid(*ks)

    knorm = np.sqrt(np.sum(np.power(kgrid,2), axis=0))
    power_k = np.where(knorm == 0, 0, np.sqrt(power_spectrum(knorm)))

    imfft = np.zeros(shape, dtype=complex)
    imfft.real = power_k * np.cos(phase)
    imfft.imag = power_k * np.sin(phase)
    
    return fft.ifftn(np.fft.ifftshift(imfft)).real


def Pkgen(n, k0, k1):
    def Pk(k):
        return np.exp(-np.power(k0/k,2)) * np.power(k, -n) * np.exp(-np.power(k/k1,2))
    return Pk

def Pkgen_break(n1, n2, kcrit, k0, k1):
    def Pk(k):
        return np.where(k < kcrit, np.exp(-np.power(k0/k,2)) * np.power(k/kcrit, -n1) * np.exp(-np.power(k/k1,2)), np.exp(-np.power(k0/k,2)) * np.power(k/kcrit, -n2) * np.exp(-np.power(k/k1,2)))
    return Pk
        

if __name__ == '__main__':    
    shape = (512,512)

    # field = fBmnd(shape, Pkgen(3,0.01,0.4), seed=31, unit_length=1)
    field_sonic = fBmnd(shape, Pkgen_break(4, 3.6, 0.08, 0.0,np.inf), seed=31, unit_length=1)

    
    
